==> free-sa-1.3.4.tar.gz: free-sa-1.3.4/src/reports/r_users.c <==
#include "global.h"


void ReportUsers(void) {
    FILE *UR = NULL, *UFR = NULL, *tUR;
    double tBytes, urlBytes = -1, *mBytes, yBytes[5][12][31][24], tlyBytes[5][12][31][24];
    short int tIndic, urlIndic = 0, i;
    int urlCount = -1, urlNumber;
    char turl[URLSIZE], oturl[URLSIZE], tfname[FILENAMESIZE], schar = '\0', *t = NULL, tmethod;
    struct tm *ttm;
    time_t ttime;
    

    if (cfg.TSR) tTS1 = SAfopen("w", "%s/%s.uns", cfg.TmpDir, icfg.TSRname);

    memset(&tlyBytes[0][0][0][0], 0, ((size_t)icfg.EYear - (size_t)icfg.SYear + 1L) * 12L * 31L * 24L * sizeof(double));
    
    for (i = 0; i < icfg.ueCount; i++) {
	if (Users[i].count) {
	    /* U_UNS: close */
	    SAfclose(Users[i].file, Users[i].fname);
	    /* U: generate U sorted file name as u_USERNAME */
	    strcpy(tfname, Users[i].fname);
	    Users[i].fname[strlen(Users[i].fname) - 4L] = '\0';
	    /* U_UNS: sort by fullurl and datetime */
	    SAsystem(CMDSORT, CMDSORT, "-k", "3,3d", "-k", "2,2n", "-o", Users[i].fname, tfname, NULL);
	    /* U_UNS: delete */
	    SAunlink(tfname);
	    
	    /* Open sorted user file, unsorted user site file u_USERNAME.uns and fullurl user report file */
	    Users[i].file = SAfopenI("r", Users[i].fname);
	    if (cfg.UR) UR = SAfopen("w", TEMPL_U_UNS, cfg.TmpDir, Users[i].name);
	    
	    if (cfg.FullURL) {
		SAsnprintf(turl, sizeof(turl), "%s, %s (%s, %s)", loc->URL, loc->DnT, loc->Asc, loc->Asc);
		UFR = SAfopenH("../", 'l', Users[i].vname, turl, 1, "w", "%s/f_%s.html", icfg.ReportDir, Users[i].name);
		SAfprintf(UFR, "<tr><th>%s<th>%s<th>%s\n", loc->URL, loc->DnT, loc->Bytes);
	    }

	    /* URL sequence number */
	    urlNumber = 0;
	    oturl[0] = '\0';

	    /* Clear user year-month-day-hour data */
	    memset(&yBytes[0][0][0][0], 0, ((size_t)icfg.EYear - (size_t)icfg.SYear + 1L) * 12L * 31L * 24L * sizeof(double));

	    /* Scan sorted by fullurl user file */
	    while(fscanf(Users[i].file, "%lf %ld %s %hd %c\n", &tBytes, &ttime, turl, &tIndic, &tmethod) == 5) {
		/* user day-time report part */
		if (cfg.UR) {
		    ttm = localtime(&ttime);
		    yBytes[ttm->tm_year - icfg.SYear][ttm->tm_mon][ttm->tm_mday - 1][ttm->tm_hour] += tBytes;
		    tlyBytes[ttm->tm_year - icfg.SYear][ttm->tm_mon][ttm->tm_mday - 1][ttm->tm_hour] += tBytes;
		}
		
		/* Shorten URL (if it's already not short due to not setted cfg.FullURL)
		(we jump over "http://" or "ftp://" here and saves replaced char) */
		if (cfg.FullURL && tmethod != 'C') {
		    for (t = &turl[7]; t[0] != '\0' && t[0] != '/'; t++);
		    schar = t[0];
		    t[0] = '\0';
		}
		
		/* If new url */
		if (strcmp(oturl, turl)) {
		    /* If not 1st iteration write to user site file and topsites unsorted file */
		    if (urlNumber) {
			if (cfg.UR) SAfprintf(UR, "%s %d %.0f %d %hd\n", oturl, urlCount, urlBytes, urlNumber, urlIndic);
			if (cfg.TSR) SAfprintf(tTS1, "%s %s %d %.0f %hd\n", oturl, Users[i].name, urlCount, urlBytes, urlIndic);
		    }
		    strcpy(oturl, turl);
		    urlNumber++;
		    urlCount = 1;
		    urlIndic = tIndic;
		    urlBytes = tBytes;
		} else {
		    urlCount++;
		    urlIndic |= tIndic;
		    urlBytes += tBytes;
		}
		
		/* write to fullurl user report file */
		if (cfg.FullURL) {
		    /* Recover full url */
		    if (tmethod != 'C') t[0] = schar;
		    
		    /* Put empty line before new site */
		    if (urlCount == 1 && urlNumber != 1) SAfwrite1("<tr><td class=el>\n", 18L, UFR);
		    SAfwrite1("<tr><td><a ", 11L, UFR);
		    if (urlCount == 1) SAfprintf(UFR, "name=\"%d\" ", urlNumber);
		    
		    SAfprintf(UFR, "href=\"%s\">%s</a><td>%s<td class=rt>%s", 
			turl, LimitedURL(turl), GetDT(ttime, "%x&nbsp;%X"), icfg.SAB(tBytes));
		    Indics(UFR, tIndic);
		}
	    }
	    
	    /* Close fullurl user report file */
	    if (cfg.FullURL) SAfcloseH(UFR, Users[i].name);

	    /* Recover short URL, write to user site file and topsites unsorted file
	       (because we use postwriting, not prewriting) */
	    if (cfg.FullURL && tmethod != 'C') t[0] = '\0'; /* ERROR HERE? NO! Check w_log_operations.c */
	    if (cfg.UR) SAfprintf(UR, "%s %d %.0f %d %hd\n", oturl, urlCount, urlBytes, urlNumber, urlIndic);
	    if (cfg.TSR) SAfprintf(tTS1, "%s %s %d %.0f %hd\n", oturl, Users[i].name, urlCount, urlBytes, urlIndic);

	    /* Close and delete sorted by fullurl user file and close user site file */
	    SAfcloseU(Users[i].file, Users[i].fname);
	    /* No work for not UR (actual for TSR) below */
	    if (!cfg.UR) continue;
	    SAfclose(UR, Users[i].name);
	    
	    /* Change names to not be messed up with same name */
	    Users[i].fname[strlen(Users[i].fname) + 1L] = '\0';
	    Users[i].fname[strlen(Users[i].fname)] = '.';
	    strcpy(tfname, Users[i].fname);
	    strcat(tfname, "uns");
	    
	    /* Sort user site file by Bytes (reverse,numeric) */
	    SAsystem(CMDSORT, CMDSORT, "-nr", "-k", "3,3", "-o", Users[i].fname, tfname, NULL);
	    /* Delete unsorted user site file */
	    SAunlink(tfname);
	    
	    /* Finally (!) write sorted data to user report */
	    tUR = SAfopenI("r", Users[i].fname);
	    SAsnprintf(turl, sizeof(turl), "%s (%s)", loc->Bytes, loc->Desc);
	    UR = SAfopenH("../", 'r', Users[i].vname, turl, 1, "w", "%s/u%s.html", icfg.ReportDir, Users[i].name);
	    
	    SAfprintf(UR, "<tr>%s<th>%s<th>%s<th>%s\n", (cfg.FullURL) ? "<th>" : "", loc->URL, loc->Connections, loc->Bytes);
	    while(fscanf(tUR, "%s %d %lf %d %hd\n", turl, &urlCount, &urlBytes, &urlNumber, &tIndic) == 5) {
		SAfwrite1("<tr>", 4L, UR);
		if (cfg.FullURL) SAfprintf(UR, "<td><a href=\"f_%s.html#%d\"><img src=\"../d.png\" alt=\"\"></a>", Users[i].name, urlNumber);
		SAfprintf(UR, "<td class=lt><a href=\"%s\">%s</a><td>%'d<td>%s", 
		    turl, turl, urlCount, icfg.SAB(urlBytes));
		Indics(UR, tIndic);
	    }
	    /* Write total string to user report */
	    SAfprintf(UR, "<tr>%s<th class=lt>%s<th class=rt>%'d<th class=rt>%s\n", (cfg.FullURL) ? "<td class=el>" : "", loc->TOTAL, Users[i].count, icfg.SAB(Users[i].Bytes));
	    
	    SAfcloseH(UR, Users[i].name);
	    SAfcloseU(tUR, Users[i].fname);
	    
	    mBytes = ReportAnyDateTime(yBytes, Users[i].vname, Users[i].name, Users[i].Bytes, 0);
	    ReportAnyGraphs(mBytes, Users[i].vname, Users[i].name);
	  /* close and delete unneeded U_UNS files */
	} else if (cfg.UR || cfg.TSR) SAfcloseU(Users[i].file, Users[i].fname);
    }
    
    /* UDR/UGR: total and average */
    mBytes = ReportAnyDateTime(tlyBytes, loc->TOTAL, icfg.TTLname, ccBytes, 1);
    ReportAnyGraphs(mBytes, loc->TOTAL, icfg.TTLname);
    
    mBytes = ReportAnyDateTime(tlyBytes, loc->AVERAGE, icfg.AVGname, ccBytes / ccueCount, 0);
    ReportAnyGraphs(mBytes, loc->AVERAGE, icfg.AVGname);
}

==> free-sa-1.4.3.tar.gz: free-sa-1.4.3/src/reports/r_users.c <==
#include "global.h"


inline void ReportAnyDTG(double (*tBytes)[12][31][24], const char *vname, const char *name, const double bytes, const char tlFlag);
inline void ReportAnyDTG(double (*tBytes)[12][31][24], const char *vname, const char *name, const double bytes, const char tlFlag) {
    short int hour, day, month, year;
    double xBytes[31];


    ReportAnyDateTime(tBytes, vname, name, bytes);
    
    memset(xBytes, 0, icfg.PScale * sizeof(double));
    switch (icfg.PScale) {
	case 12:
		for (year = 0; year <= icfg.EYear - icfg.SYear; year++)
		    for (month = 0; month < 12; month++)
		        for (day = 0; day < 31; day++)
    			    for (hour = 0; hour < 24; hour++) {
    				if (tBytes[year][month][day][hour]) {
				    xBytes[month] += tBytes[year][month][day][hour];
				    if (tlFlag) tBytes[year][month][day][hour] = tBytes[year][month][day][hour] / ccueCount;
				}
			    }
		break;
	case 24:
		for (year = 0; year <= icfg.EYear - icfg.SYear; year++)
		    for (month = 0; month < 12; month++)
		        for (day = 0; day < 31; day++)
    			    for (hour = 0; hour < 24; hour++) {
    			    	if (tBytes[year][month][day][hour]) {
			    	    xBytes[hour] += tBytes[year][month][day][hour];
				    if (tlFlag) tBytes[year][month][day][hour] = tBytes[year][month][day][hour] / ccueCount;
				}
			    }
		break;
	case 31:
		for (year = 0; year <= icfg.EYear - icfg.SYear; year++)
		    for (month = 0; month < 12; month++)
		        for (day = 0; day < 31; day++)
    			    for (hour = 0; hour < 24; hour++) {
    				if (tBytes[year][month][day][hour]) {
				    xBytes[day] += tBytes[year][month][day][hour];
				    if (tlFlag) tBytes[year][month][day][hour] = tBytes[year][month][day][hour] / ccueCount;
				}
			    }
		break;
    }
    
    ReportAnyGraphs(xBytes, vname, name, icfg.PScale);
}


void ReportUsers(void) {
    FILE *UR = NULL, *UFR = NULL, *tUR;
    double tBytes, urlBytes = -1, yBytes[5][12][31][24], tlyBytes[5][12][31][24];
    short int tIndic, urlIndic = 0, i;
    int urlCount = -1, urlNumber;
    char turl[URLSIZE], oturl[URLSIZE], tfname[FILENAMESIZE], schar = '\0', *t = NULL, tmethod;
    struct tm *ttm;
    time_t ttime;
    

    if (cfg.TSR) tTS1 = SAfopen("w", "%s/%s.uns", cfg.TmpDir, icfg.TSRname);

    memset(&tlyBytes[0][0][0][0], 0, ((size_t)icfg.EYear - (size_t)icfg.SYear + 1L) * 12L * 31L * 24L * sizeof(double));
    
    for (i = 0; i < icfg.ueCount; i++) {
	if (Users[i].count) {
	    /* U_UNS: close */
	    SAfclose(Users[i].file, Users[i].fname);
	    /* U: generate U sorted file name as u_USERNAME */
	    strcpy(tfname, Users[i].fname);
	    Users[i].fname[strlen(Users[i].fname) - 4L] = '\0';
	    /* U_UNS: sort by fullurl and datetime */
	    SAsystem(CMDSORT, CMDSORT, "-k", "3,3d", "-k", "2,2n", "-o", Users[i].fname, tfname, NULL);
	    /* U_UNS: delete */
	    SAunlink(tfname);
	    
	    /* Open sorted user file, unsorted user site file u_USERNAME.uns and fullurl user report file */
	    Users[i].file = SAfopenI("r", Users[i].fname);
	    if (cfg.UR) UR = SAfopen("w", TEMPL_U_UNS, cfg.TmpDir, Users[i].name);
	    
	    if (cfg.FullURL) {
		SAsnprintf(turl, sizeof(turl), "%s, %s (%s, %s)", loc->URL, loc->DnT, loc->Asc, loc->Asc);
		UFR = SAfopenH("../", 'l', Users[i].vname, turl, 1, "w", "%s/f_%s.html", icfg.ReportDir, Users[i].name);
		SAfprintf(UFR, "<tr><th>%s<th>%s<th>%s\n", loc->URL, loc->DnT, loc->Bytes);
	    }

	    /* URL sequence number */
	    urlNumber = 0;
	    oturl[0] = '\0';

	    /* Clear user year-month-day-hour data */
	    memset(&yBytes[0][0][0][0], 0, ((size_t)icfg.EYear - (size_t)icfg.SYear + 1L) * 12L * 31L * 24L * sizeof(double));

	    /* Scan sorted by fullurl user file */
	    while(fscanf(Users[i].file, "%lf %ld %s %hd %c\n", &tBytes, &ttime, turl, &tIndic, &tmethod) == 5) {
		/* user day-time report part */
		if (cfg.UR) {
		    ttm = localtime(&ttime);
		    yBytes[ttm->tm_year - icfg.SYear][ttm->tm_mon][ttm->tm_mday - 1][ttm->tm_hour] += tBytes;
		    tlyBytes[ttm->tm_year - icfg.SYear][ttm->tm_mon][ttm->tm_mday - 1][ttm->tm_hour] += tBytes;
		}
		
		/* Shorten URL (if it's already not short due to not setted cfg.FullURL)
		(we jump over "http://" or "ftp://" here and saves replaced char) */
		if (cfg.FullURL && tmethod != 'C') {
		    for (t = &turl[7]; t[0] != '\0' && t[0] != '/'; t++);
		    schar = t[0];
		    t[0] = '\0';
		}
		
		/* If new url */
		if (strcmp(oturl, turl)) {
		    /* If not 1st iteration write to user site file and topsites unsorted file */
		    if (urlNumber) {
			if (cfg.UR) SAfprintf(UR, "%s %d %.0f %d %hd\n", oturl, urlCount, urlBytes, urlNumber, urlIndic);
			if (cfg.TSR) SAfprintf(tTS1, "%s %s %d %.0f %hd\n", oturl, Users[i].name, urlCount, urlBytes, urlIndic);
		    }
		    strcpy(oturl, turl);
		    urlNumber++;
		    urlCount = 1;
		    urlIndic = tIndic;
		    urlBytes = tBytes;
		} else {
		    urlCount++;
		    urlIndic |= tIndic;
		    urlBytes += tBytes;
		}
		
		/* write to fullurl user report file */
		if (cfg.FullURL) {
		    /* Recover full url */
		    if (tmethod != 'C') t[0] = schar;
		    
		    /* Put empty line before new site */
		    if (urlCount == 1 && urlNumber != 1) SAfwrite1("<tr><td class=el>\n", 18L, UFR);
		    SAfwrite1("<tr><td><a ", 11L, UFR);
		    if (urlCount == 1) SAfprintf(UFR, "name=\"%d\" ", urlNumber);
		    
		    SAfprintf(UFR, "href=\"%s\">%s</a><td>%s<td class=rt>%s", 
			turl, LimitedURL(turl), GetDT(ttime, "%x&nbsp;%X"), icfg.SAB(tBytes));
		    Indics(UFR, tIndic);
		}
	    }
	    
	    /* Close fullurl user report file */
	    if (cfg.FullURL) SAfcloseH(UFR, Users[i].name);

	    /* Recover short URL, write to user site file and topsites unsorted file
	       (because we use postwriting, not prewriting) */
	    if (cfg.FullURL && tmethod != 'C') t[0] = '\0'; /* ERROR HERE? NO! Check w_log_operations.c */
	    if (cfg.UR) SAfprintf(UR, "%s %d %.0f %d %hd\n", oturl, urlCount, urlBytes, urlNumber, urlIndic);
	    if (cfg.TSR) SAfprintf(tTS1, "%s %s %d %.0f %hd\n", oturl, Users[i].name, urlCount, urlBytes, urlIndic);

	    /* Close and delete sorted by fullurl user file and close user site file */
	    SAfcloseU(Users[i].file, Users[i].fname);
	    /* No work for not UR (actual for TSR) below */
	    if (!cfg.UR) continue;
	    SAfclose(UR, Users[i].name);
	    
	    /* Change names to not be messed up with same name */
	    Users[i].fname[strlen(Users[i].fname) + 1L] = '\0';
	    Users[i].fname[strlen(Users[i].fname)] = '.';
	    strcpy(tfname, Users[i].fname);
	    strcat(tfname, "uns");
	    
	    /* Sort user site file by Bytes (reverse,numeric) */
	    SAsystem(CMDSORT, CMDSORT, "-nr", "-k", "3,3", "-o", Users[i].fname, tfname, NULL);
	    /* Delete unsorted user site file */
	    SAunlink(tfname);
	    
	    /* Finally (!) write sorted data to user report */
	    tUR = SAfopenI("r", Users[i].fname);
	    SAsnprintf(turl, sizeof(turl), "%s (%s)", loc->Bytes, loc->Desc);
	    UR = SAfopenH("../", 'r', Users[i].vname, turl, 1, "w", "%s/u%s.html", icfg.ReportDir, Users[i].name);
	    
	    SAfprintf(UR, "<tr>%s<th>%s<th>%s<th>%s\n", (cfg.FullURL) ? "<th>" : "", loc->URL, loc->Connections, loc->Bytes);
	    while(fscanf(tUR, "%s %d %lf %d %hd\n", turl, &urlCount, &urlBytes, &urlNumber, &tIndic) == 5) {
		SAfwrite1("<tr>", 4L, UR);
		if (cfg.FullURL) SAfprintf(UR, "<td><a href=\"f_%s.html#%d\"><img src=\"../d.png\" alt=\"\"></a>", Users[i].name, urlNumber);
		SAfprintf(UR, "<td class=lt><a href=\"%s\">%s</a><td>%'d<td>%s", 
		    turl, turl, urlCount, icfg.SAB(urlBytes));
		Indics(UR, tIndic);
	    }
	    /* Write total string to user report */
	    SAfprintf(UR, "<tr>%s<th class=lt>%s<th class=rt>%'d<th class=rt>%s\n", (cfg.FullURL) ? "<td class=el>" : "", loc->TOTAL, Users[i].count, icfg.SAB(Users[i].Bytes));
	    
	    SAfcloseH(UR, Users[i].name);
	    SAfcloseU(tUR, Users[i].fname);
	    
	    ReportAnyDTG(yBytes, Users[i].vname, Users[i].name, Users[i].Bytes, 0);
	  /* close and delete unneeded U_UNS files */
	} else if (cfg.UR || cfg.TSR) SAfcloseU(Users[i].file, Users[i].fname);
    }
    
    /* UDR/UGR: total and average */
    ReportAnyDTG(tlyBytes, loc->TOTAL, icfg.TTLname, ccBytes, 1);
    ReportAnyDTG(tlyBytes, loc->AVERAGE, icfg.AVGname, ccBytes / ccueCount, 0);
}
